import datetime
import os
import zoneinfo

import icalendar
import yaml

from .base import ImportBase


class ImportICAL(ImportBase):

    def go(self, group: dict):
        os.makedirs(
            os.path.join(self._settings.DATA_DIRECTORY, "event", group["id"]),
            exist_ok=True,
        )
        with open(self._download_file_to_temp(group["field_import_url"])) as fp:
            calendar = icalendar.Calendar.from_ical(fp.read())
            for event in calendar.events:  # type: ignore
                timezone_name = group["field_timezone"] or "UTC"
                start_datetime = event.get("DTSTART").dt
                end_datetime = event.get("DTEND").dt
                if isinstance(start_datetime, datetime.datetime):
                    pass
                elif isinstance(start_datetime, datetime.date):
                    start_datetime = datetime.datetime(
                        start_datetime.year,
                        start_datetime.month,
                        start_datetime.day,
                        0,
                        0,
                        0,
                        tzinfo=zoneinfo.ZoneInfo(timezone_name),
                    )
                else:
                    raise Exception(
                        "Start not in the format we expect {}".format(start_datetime)
                    )
                if isinstance(end_datetime, datetime.datetime):
                    pass
                elif isinstance(end_datetime, datetime.date):
                    end_datetime = datetime.datetime(
                        end_datetime.year,
                        end_datetime.month,
                        end_datetime.day,
                        23,
                        59,
                        59,
                        tzinfo=zoneinfo.ZoneInfo(timezone_name),
                    )
                else:
                    raise Exception(
                        "End not in the format we expect {}".format(end_datetime)
                    )
                if self._should_import_event(
                    group,
                    str(event.get("SUMMARY")),
                    str(event.get("DESCRIPTION")),
                    end_datetime,
                ):
                    # Create event data with various fields
                    event_data = {
                        "title": str(event.get("SUMMARY")),
                        "group": group["id"],
                        "timezone": timezone_name,
                        "start_at": str(start_datetime),
                        "end_at": str(end_datetime),
                        "url": str(event.get("URL", "")) or group["field_url"],
                        "cancelled": (event.get("STATUS") == "CANCELLED"),
                        "imported": True,
                        "community_participation": {
                            "at_event": "unknown",
                            "at_event_audience_text": "unknown",
                            "at_event_audience_audio": "unknown",
                        },
                        "in_person": "unknown",
                    }
                    # Add data from group
                    self._update_event_with_group_data(event_data, group)
                    # Id
                    id = event.get("UID").split("@").pop(0)
                    # filename
                    filename = os.path.join(
                        self._settings.DATA_DIRECTORY, "event", group["id"], id + ".md"
                    )
                    # Finally write data
                    with open(filename, "w") as fp:
                        fp.write("---\n")
                        fp.write(yaml.dump(event_data))
                        fp.write("---\n\n\n")
                        fp.write(event.get("DESCRIPTION"))
                        fp.write("\n")



import os

import requests
import requests_cache


class Settings:

    def __init__(self):

        self.USER_AGENT = (
            "Open Tech Calendar Tools https://opentechcalendar.co.uk/contact"
        )

        # TODO It's not really a directory, filename would be a better descriptor
        requests_cache_directory = os.getenv(
            "OPEN_TECH_CALENDAR_TOOLS_REQUEST_CACHE_DIRECTORY",
        )
        self.REQUESTS_SESSION = (
            requests_cache.CachedSession(requests_cache_directory)
            if requests_cache_directory
            else requests.Session()
        )
        self.DATA_DIRECTORY = os.getcwd()

        self.SQLITE_DATABASE_FILENAME = os.getenv(
            "OPEN_TECH_CALENDAR_TOOLS_SQLITE_DATABASE_FILENAME"
        )



import sqlite3

import opentechcalendartools.importers.eventbrite_organisation
import opentechcalendartools.importers.ical
from opentechcalendartools.settings import Settings


class Worker:

    def __init__(self, settings: Settings):
        self._settings: Settings = settings

    def get_group_ids_to_import(self) -> list:
        with sqlite3.connect(self._settings.SQLITE_DATABASE_FILENAME) as connection:
            res = connection.cursor().execute(
                "SELECT id FROM record_group WHERE field_import_type != '' AND field_import_type IS NOT NULL ORDER BY id ASC"
            )
            return [i[0] for i in res.fetchall()]

    def import_group(self, group_id):
        with sqlite3.connect(self._settings.SQLITE_DATABASE_FILENAME) as connection:
            connection.row_factory = sqlite3.Row
            cursor = connection.cursor()
            res = cursor.execute("SELECT * FROM record_group WHERE id=?", [group_id])
            group = res.fetchone()
            if group["field_import_type"] == "ical":
                importer = opentechcalendartools.importers.ical.ImportICAL(
                    self._settings
                )
                importer.go(group)
            elif group["field_import_type"] == "eventbrite-organisation":
                importer = opentechcalendartools.importers.eventbrite_organisation.ImportEventbriteOrganisation(
                    self._settings
                )
                importer.go(group)



import argparse

from opentechcalendartools.settings import Settings

from .worker import Worker


def main():

    # First we process the data files into a SQLite database for us to use
    settings = Settings()
    if not settings.SQLITE_DATABASE_FILENAME:
        raise Exception(
            "Must specify OPEN_TECH_CALENDAR_TOOLS_SQLITE_DATABASE_FILENAME env var"
        )
        # TODO better would be to open a tempfile ourselves instead, and build it

    # Now check options ...
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(dest="subparser_name")

    subparsers.add_parser("listgroupstoimport")

    import_group_parser = subparsers.add_parser("importgroup")
    import_group_parser.add_argument("group_id")

    args = parser.parse_args()

    worker = Worker(settings)

    if args.subparser_name == "listgroupstoimport":
        # List groups to import
        for group_id in worker.get_group_ids_to_import():
            print(group_id)

    elif args.subparser_name == "importgroup":
        # Import Group
        worker.import_group(args.group_id)



import setuptools

setuptools.setup(
    name="opentechcalendartools",
    version="0.5.0",
    description="",
    url="https://github.com/TeacakeTech/opentechcalendar-tools",
    project_urls={
        "Home Page": "https://opentechcalendar.co.uk/",
        "Issues": "https://github.com/TeacakeTech/opentechcalendar-tools/issues",
        "Source": "https://github.com/TeacakeTech/opentechcalendar-tools",
    },
    author="Teacake Tech",
    author_email="hello@teacaketech.scot",
    packages=setuptools.find_packages(exclude=["test"]),
    install_requires=[
        "datatig",
        "requests",
        "requests-cache",
        "icalendar",
        "pyyaml",
        "beautifulsoup4",
    ],
    extras_require={
        "dev": [
            "black",
            "isort",
            "flake8",
            "mypy",
        ],
    },
    python_requires=">=3.12",
)



import datetime
import json
import os

import yaml
from bs4 import BeautifulSoup

from .base import ImportBase


class ImportEventbriteOrganisation(ImportBase):

    def go(self, group: dict):
        os.makedirs(
            os.path.join(self._settings.DATA_DIRECTORY, "event", group["id"]),
            exist_ok=True,
        )
        with open(self._download_file_to_temp(group["field_import_url"])) as fp:
            soup = BeautifulSoup(fp.read(), "html.parser")

            datas = [
                json.loads(s.string)  # type: ignore
                for s in soup.find_all("script", attrs={"type": "application/ld+json"})
            ]
            datas = [d for d in datas if isinstance(d.get("itemListElement"), list)]

            for event_jsonld_data in datas[0]["itemListElement"]:
                if event_jsonld_data["@type"] != "ListItem":
                    continue
                if event_jsonld_data["item"]["@type"] != "Event":
                    continue
                if event_jsonld_data["item"]["eventAttendanceMode"] not in [
                    "https://schema.org/OnlineEventAttendanceMode",
                    "https://schema.org/MixedEventAttendanceMode",
                ]:
                    continue
                timezone_name = group["field_timezone"] or "UTC"
                start_datetime = datetime.datetime.fromisoformat(
                    event_jsonld_data["item"]["startDate"]
                )
                end_datetime = datetime.datetime.fromisoformat(
                    event_jsonld_data["item"]["startDate"]
                )
                if self._should_import_event(
                    group,
                    event_jsonld_data["item"]["name"],
                    event_jsonld_data["item"]["description"],
                    end_datetime,
                ):

                    # Create event data with various fields
                    event_data = {
                        "title": event_jsonld_data["item"]["name"],
                        "group": group["id"],
                        "timezone": timezone_name,
                        "start_at": str(start_datetime),
                        "end_at": str(end_datetime),
                        "url": event_jsonld_data["item"]["url"],
                        "cancelled": False,  # TODO
                        "imported": True,
                        "community_participation": {
                            "at_event": "unknown",
                            "at_event_audience_text": "unknown",
                            "at_event_audience_audio": "unknown",
                        },
                        "in_person": "unknown",
                    }
                    # Add data from group
                    self._update_event_with_group_data(event_data, group)
                    # Overwrite group data with some extra data we know
                    if (
                        event_jsonld_data["item"]["eventAttendanceMode"]
                        == "https://schema.org/OnlineEventAttendanceMode"
                    ):
                        event_data["in_person"] = "no"
                    elif (
                        event_jsonld_data["item"]["eventAttendanceMode"]
                        == "https://schema.org/MixedEventAttendanceMode"
                    ):
                        event_data["in_person"] = "yes"
                    # Id
                    id = event_data["url"].split("/").pop(-1)
                    # filename
                    filename = os.path.join(
                        self._settings.DATA_DIRECTORY, "event", group["id"], id + ".md"
                    )
                    # Finally write data
                    with open(filename, "w") as fp:
                        fp.write("---\n")
                        fp.write(yaml.dump(event_data))
                        fp.write("---\n\n\n")
                        fp.write(event_jsonld_data["item"]["description"])
                        fp.write("\n")



import datetime
import os
import sqlite3
import tempfile

from opentechcalendartools.settings import Settings


class ImportBase:

    def __init__(self, settings: Settings):
        self._settings: Settings = settings

    def _download_file_to_temp(self, url) -> str:
        r = self._settings.REQUESTS_SESSION.get(
            url,
            headers={"User-Agent": self._settings.USER_AGENT},
        )
        r.raise_for_status()
        new_filename_dets = tempfile.mkstemp(
            suffix="opentechcalendartools_",
        )
        os.write(new_filename_dets[0], r.content)
        os.close(new_filename_dets[0])
        return new_filename_dets[1]

    def _update_event_with_group_data(self, event_data: dict, group: dict):
        if group["field_country"]:
            event_data["country"] = group["field_country"]
        if group["field_place"]:
            event_data["place"] = group["field_place"]
        if group["field_code_of_conduct_url"]:
            event_data["code_of_conduct_url"] = group["field_code_of_conduct_url"]
        # In-person events
        if group["field_in_person"] == "all":
            event_data["in_person"] = "yes"
        elif group["field_in_person"] == "none":
            event_data["in_person"] = "no"
        # Community Participation: Interact with event?
        if group["field_community_participation_at_event"] == "all":
            event_data["community_participation"]["at_event"] = "yes"
        elif group["field_community_participation_at_event"] == "none":
            event_data["community_participation"]["at_event"] = "no"
        # Community Participation: Interact with other audience members at the event via text?
        if group["field_community_participation_at_event_audience_text"] == "all":
            event_data["community_participation"]["at_event_audience_text"] = "yes"
        elif group["field_community_participation_at_event_audience_text"] == "none":
            event_data["community_participation"]["at_event_audience_text"] = "no"
        # Community Participation: Interact with other audience members at the event via audio?
        if group["field_community_participation_at_event_audience_audio"] == "all":
            event_data["community_participation"]["at_event_audience_audio"] = "yes"
        elif group["field_community_participation_at_event_audience_audio"] == "none":
            event_data["community_participation"]["at_event_audience_audio"] = "no"

    def _should_import_event(
        self, group: dict, title: str, description: str, end: datetime.datetime
    ) -> bool:
        # Has event passed?
        if end.timestamp() < datetime.datetime.now().timestamp():
            return False

        with sqlite3.connect(self._settings.SQLITE_DATABASE_FILENAME) as connection:
            # Check Excludes
            connection.row_factory = sqlite3.Row
            cursor = connection.cursor()
            for row in cursor.execute(
                "SELECT * FROM record_group___field_import_exclude WHERE record_id=?",
                [group["id"]],
            ):
                if row["value"] in title or row["value"] in description:
                    return False

            # Check includes
            any_includes = False
            for row in cursor.execute(
                "SELECT * FROM record_group___field_import_include WHERE record_id=?",
                [group["id"]],
            ):
                if row["value"] in title or row["value"] in description:
                    return True
                any_includes = True
            # If there were include values set but none of them matched then ...
            if any_includes:
                return False

        # Ok, have checked the excludes and there were no includes to check, so default to ....
        return True

    def go(self, group: dict):
        pass
